﻿using System;
using System.Collections.Generic;

using BlackBerry.Workspaces.Json;
using BlackBerry.Workspaces.Resource;

namespace BlackBerry.Workspaces.Examples
{
    /// <summary>
    /// Examples of Workspaces User functionality
    /// </summary>
    public class UsersExample
    {
        private ApiSession apiSession;

        /// <summary>
        /// Creates an instances of the UsersExample class
        /// </summary>
        /// <param name="apiSession">
        /// An ApiSession instance to be used.
        /// </param>
        public UsersExample(ApiSession apiSession)
        {
            this.apiSession = apiSession;
        }

        /// <summary>
        /// Example to add a specific user.
        /// </summary>
        /// <param name="userEmail">
        /// The user's email address which will be used as the username
        /// </param>
        /// <param name="aliases">
        /// A set of aliases for the user
        /// </param>
        /// <param name="roles">
        /// A set of Workspaces roles for the user.
        /// </param>
        /// <param name="friendlyName">
        /// The user's name
        /// </param>
        /// <returns>
        /// An object describing the result of the operation
        /// </returns>
        public BulkOperationResultJson AddUser(String userEmail, HashSet<String> aliases, HashSet<String> roles,
                                               String friendlyName)
        {
            Organizations organizations = apiSession.GetOrganizationsResource();
            UsersJson usersJson = new UsersJson();
            UserJson userJson = new UserJson();
            UserDetailsJson userDetailsJson = new UserDetailsJson();
            List<UserJson> userJsonList = new List<UserJson>();

            userDetailsJson.Aliases = aliases;
            userDetailsJson.Roles = roles;
            userDetailsJson.UserName = friendlyName;
            userJson.UserEmail = userEmail;
            userJson.UserDetails = userDetailsJson;
            userJsonList.Add(userJson);
            usersJson.Users = userJsonList;

            return organizations.AddUsersV30(usersJson);
        }

        /// <summary>
        /// Example to edit a specific user.
        /// </summary>
        /// <param name="userName">
        /// The username for a user
        /// </param>
        /// <param name="language">
        /// A new language for a user
        /// </param>
        /// <param name="sendNotifications">
        /// A flag indicating whether notifications should be sent
        /// </param>
        /// <param name="timezoneOffset">
        /// A new timezone offset for a user.
        /// </param>
        /// <returns>
        /// An object describing the result of the operation
        /// </returns>
        public BulkOperationResultJson EditUser(String userName, String language, Boolean sendNotifications, int timezoneOffset)
        {
            Users users = apiSession.GetUsersResource();
            EditUserJson editUserJson = new EditUserJson
            {
                Language = language,
                SendNotifications = sendNotifications,
                UpdateSendNotifications = true,
                TimezoneOffset = timezoneOffset,
                UpdateTimezoneOffset = true,
                UserName = userName
            };

            return users.EditUserV30(editUserJson);
        }

        /// <summary>
        /// Example to remove users.
        /// </summary>
        /// <param name="usernames">
        /// A list of usernames to be removed.</param>
        /// <returns>
        /// An object describing the result of the operation
        /// </returns>
        public BulkOperationResultJson RemoveUsers(List<String> usernames)
        {
            Organizations organizations = apiSession.GetOrganizationsResource();
            RemoveUsersJson removeUsersJson = new RemoveUsersJson
            {
                ForceAction = true,
                Users = usernames
            };

            return organizations.DeleteUsersV30(removeUsersJson);
        }

        /// <summary>
        /// Example to copy membership( of several users ) to another existing user.
        /// </summary>
        /// <param name="usersFrom">
        /// A list of usernames for the users that will have their membership copied.
        /// </param>
        /// <param name="userTo">
        /// The username to receive the membership
        /// </param>
        /// <returns>
        /// An object describing the result of the operation
        /// </returns>
        public BulkOperationResultJson CopyMemebership(List<String> usersFrom, String userTo)
        {
            Organizations organizations = apiSession.GetOrganizationsResource();
            CopyMembershipJson copyMembershipJson = new CopyMembershipJson
            {
                ForceAction = true,
                UsersFrom = usersFrom,
                UserTo = userTo
            };

            return organizations.CopyMembershipV30(copyMembershipJson);
        }

    }
}
